export declare function lighten(c: string, amount: number): string;
export declare function alpha(c: string, amount: number): string;
/**
 * normalize open-color to remove some colours,
 * such as 'grape'. This should be compatible with a tool
 * like palx.
 */
declare const defaultScales: {
    gray: string[];
    blue: string[];
    green: string[];
    red: string[];
    orange: string[];
    yellow: string[];
    teal: string[];
    cyan: string[];
    lime: string[];
    pink: string[];
    violet: string[];
    indigo: string[];
};
export declare type ScalesType = typeof defaultScales;
declare type PaletteItem = {
    lightest: string;
    light: string;
    base: string;
    dark: string;
};
export declare type PaletteType = {
    gray: PaletteItem;
    blue: PaletteItem;
    red: PaletteItem;
    orange: PaletteItem;
    yellow: PaletteItem;
    green: PaletteItem;
    teal: PaletteItem;
    violet: PaletteItem;
};
/**
 * Generate a palette from scales.
 *
 * `light` is typically used in dark mode for things like
 *  outline buttons, ghost buttons, etc.
 *
 * `base` is typically used for buttons, avatar colors, etc.
 *
 * @param scales
 */
declare function defaultGeneratePalette(scales: ScalesType): PaletteType;
/**
 * Generate lightmode colors
 * @param scales
 * @param palette
 */
declare function defaultGenerateLightMode(scales: ScalesType, palette: PaletteType): {
    background: {
        tint1: string;
        tint2: string;
        overlay: string;
        layer: string;
        default: string;
    };
    border: {
        default: string;
        muted: string;
    };
    text: {
        heading: string;
        muted: string;
        default: string;
        selected: string;
    };
    shadows: {
        xs: string;
        sm: string;
        md: string;
        lg: string;
        xl: string;
    };
};
/**
 * Generate dark mode colors
 * @param scales
 * @param palette
 */
declare function defaultGenerateDarkMode(scales: ScalesType, palette: PaletteType): {
    background: {
        tint1: string;
        tint2: string;
        overlay: string;
        layer: string;
        default: string;
    };
    border: {
        default: string;
        muted: string;
    };
    text: {
        heading: string;
        muted: string;
        default: string;
        selected: string;
    };
    shadows: {
        xs: string;
        sm: string;
        md: string;
        lg: string;
        xl: string;
    };
};
/**
 * Intents map a color palette to a particular intent (ie, primary, success)
 * @param palette
 */
declare function defaultGenerateIntents(palette: PaletteType): {
    none: PaletteItem;
    primary: PaletteItem;
    success: PaletteItem;
    danger: PaletteItem;
    warning: PaletteItem;
};
/**
 * You can override the default generators by
 * providing them as a second argument. This is useful for
 * providing custom intents (primary === teal, maybe)
 */
interface Builders {
    generateIntents?: typeof defaultGenerateIntents;
    generatePalette?: typeof defaultGeneratePalette;
    generateLightMode?: typeof defaultGenerateLightMode;
    generateDarkMode?: typeof defaultGenerateDarkMode;
}
export declare function generateColorsFromScales(scales: ScalesType, generators?: Builders): {
    colors: {
        palette: PaletteType;
        scales: {
            gray: string[];
            blue: string[];
            green: string[];
            red: string[];
            orange: string[];
            yellow: string[];
            teal: string[];
            cyan: string[];
            lime: string[];
            pink: string[];
            violet: string[];
            indigo: string[];
        };
        intent: {
            none: PaletteItem;
            primary: PaletteItem;
            success: PaletteItem;
            danger: PaletteItem;
            warning: PaletteItem;
        };
        background: {
            tint1: string;
            tint2: string;
            overlay: string;
            layer: string;
            default: string;
        };
        border: {
            default: string;
            muted: string;
        };
        text: {
            heading: string;
            muted: string;
            default: string;
            selected: string;
        };
        shadows: {
            xs: string;
            sm: string;
            md: string;
            lg: string;
            xl: string;
        };
        mode: string;
    };
    modes: {
        light: {
            palette: PaletteType;
            scales: {
                gray: string[];
                blue: string[];
                green: string[];
                red: string[];
                orange: string[];
                yellow: string[];
                teal: string[];
                cyan: string[];
                lime: string[];
                pink: string[];
                violet: string[];
                indigo: string[];
            };
            intent: {
                none: PaletteItem;
                primary: PaletteItem;
                success: PaletteItem;
                danger: PaletteItem;
                warning: PaletteItem;
            };
            background: {
                tint1: string;
                tint2: string;
                overlay: string;
                layer: string;
                default: string;
            };
            border: {
                default: string;
                muted: string;
            };
            text: {
                heading: string;
                muted: string;
                default: string;
                selected: string;
            };
            shadows: {
                xs: string;
                sm: string;
                md: string;
                lg: string;
                xl: string;
            };
            mode: string;
        };
        dark: {
            palette: PaletteType;
            scales: {
                gray: string[];
                blue: string[];
                green: string[];
                red: string[];
                orange: string[];
                yellow: string[];
                teal: string[];
                cyan: string[];
                lime: string[];
                pink: string[];
                violet: string[];
                indigo: string[];
            };
            intent: {
                none: PaletteItem;
                primary: PaletteItem;
                success: PaletteItem;
                danger: PaletteItem;
                warning: PaletteItem;
            };
            background: {
                tint1: string;
                tint2: string;
                overlay: string;
                layer: string;
                default: string;
            };
            border: {
                default: string;
                muted: string;
            };
            text: {
                heading: string;
                muted: string;
                default: string;
                selected: string;
            };
            shadows: {
                xs: string;
                sm: string;
                md: string;
                lg: string;
                xl: string;
            };
            mode: string;
        };
    };
};
export declare const defaultColors: {
    colors: {
        palette: PaletteType;
        scales: {
            gray: string[];
            blue: string[];
            green: string[];
            red: string[];
            orange: string[];
            yellow: string[];
            teal: string[];
            cyan: string[];
            lime: string[];
            pink: string[];
            violet: string[];
            indigo: string[];
        };
        intent: {
            none: PaletteItem;
            primary: PaletteItem;
            success: PaletteItem;
            danger: PaletteItem;
            warning: PaletteItem;
        };
        background: {
            tint1: string;
            tint2: string;
            overlay: string;
            layer: string;
            default: string;
        };
        border: {
            default: string;
            muted: string;
        };
        text: {
            heading: string;
            muted: string;
            default: string;
            selected: string;
        };
        shadows: {
            xs: string;
            sm: string;
            md: string;
            lg: string;
            xl: string;
        };
        mode: string;
    };
    modes: {
        light: {
            palette: PaletteType;
            scales: {
                gray: string[];
                blue: string[];
                green: string[];
                red: string[];
                orange: string[];
                yellow: string[];
                teal: string[];
                cyan: string[];
                lime: string[];
                pink: string[];
                violet: string[];
                indigo: string[];
            };
            intent: {
                none: PaletteItem;
                primary: PaletteItem;
                success: PaletteItem;
                danger: PaletteItem;
                warning: PaletteItem;
            };
            background: {
                tint1: string;
                tint2: string;
                overlay: string;
                layer: string;
                default: string;
            };
            border: {
                default: string;
                muted: string;
            };
            text: {
                heading: string;
                muted: string;
                default: string;
                selected: string;
            };
            shadows: {
                xs: string;
                sm: string;
                md: string;
                lg: string;
                xl: string;
            };
            mode: string;
        };
        dark: {
            palette: PaletteType;
            scales: {
                gray: string[];
                blue: string[];
                green: string[];
                red: string[];
                orange: string[];
                yellow: string[];
                teal: string[];
                cyan: string[];
                lime: string[];
                pink: string[];
                violet: string[];
                indigo: string[];
            };
            intent: {
                none: PaletteItem;
                primary: PaletteItem;
                success: PaletteItem;
                danger: PaletteItem;
                warning: PaletteItem;
            };
            background: {
                tint1: string;
                tint2: string;
                overlay: string;
                layer: string;
                default: string;
            };
            border: {
                default: string;
                muted: string;
            };
            text: {
                heading: string;
                muted: string;
                default: string;
                selected: string;
            };
            shadows: {
                xs: string;
                sm: string;
                md: string;
                lg: string;
                xl: string;
            };
            mode: string;
        };
    };
};
export {};
